@@ -87,7 +87,10 @@ cm_cell_t *cm_UpdateCell(cm_cell_t * cp, afs_uint32 flags)
         || (cm_dnsEnabled && (cp->flags & CM_CELLFLAG_DNS) &&
          ((cp->flags & CM_CELLFLAG_VLSERVER_INVALID)))
 #endif
-            ) {
+            ) 
+    {
+        lock_ReleaseMutex(&cp->mx);
+
         /* must empty cp->vlServersp */
         if (cp->vlServersp) {
             cm_FreeServerList(&cp->vlServersp, CM_FREESERVERLIST_DELETE);
@@ -97,16 +100,23 @@ cm_cell_t *cm_UpdateCell(cm_cell_t * cp, afs_uint32 flags)
         rock.cellp = cp;
         rock.flags = flags;
         code = cm_SearchCellFile(cp->name, NULL, cm_AddCellProc, &rock);
+        if (code == 0) {
+            lock_ObtainMutex(&cp->mx);
+	    cp->timeout = time(0) + 7200;
+            lock_ReleaseMutex(&cp->mx);
+        }
 #ifdef AFS_AFSDB_ENV
-        if (code) {
+        else {
             if (cm_dnsEnabled) {
                 int ttl;
 
                 code = cm_SearchCellByDNS(cp->name, NULL, &ttl, cm_AddCellProc, &rock);
                 if (code == 0) {   /* got cell from DNS */
+                    lock_ObtainMutex(&cp->mx);
                     cp->flags |= CM_CELLFLAG_DNS;
                     cp->flags &= ~CM_CELLFLAG_VLSERVER_INVALID;
 		    cp->timeout = time(0) + ttl;
+                    lock_ReleaseMutex(&cp->mx);
 #ifdef DEBUG
                     fprintf(stderr, "cell %s: ttl=%d\n", cp->name, ttl);
 #endif
@@ -114,16 +124,16 @@ cm_cell_t *cm_UpdateCell(cm_cell_t * cp, afs_uint32 flags)
                     /* if we fail to find it this time, we'll just do nothing and leave the
                      * current entry alone 
 		     */
+                    lock_ObtainMutex(&cp->mx);
                     cp->flags |= CM_CELLFLAG_VLSERVER_INVALID;
+                    lock_ReleaseMutex(&cp->mx);
                 }
 	    }
-	} else 
+	}
 #endif /* AFS_AFSDB_ENV */
-	{
-	    cp->timeout = time(0) + 7200;
-	}	
+    } else {
+        lock_ReleaseMutex(&cp->mx);
     }
-    lock_ReleaseMutex(&cp->mx);
     return code ? NULL : cp;
 }
 
@@ -133,12 +143,23 @@ cm_cell_t *cm_GetCell(char *namep, afs_uint32 flags)
     return cm_GetCell_Gen(namep, NULL, flags);
 }
 
+void cm_FreeCell(cm_cell_t *cellp)
+{
+    if (cellp->vlServersp)
+        cm_FreeServerList(&cellp->vlServersp, CM_FREESERVERLIST_DELETE);
+    cellp->name[0] = '\0';    
+
+    cellp->freeNextp = cm_data.freeCellsp;
+    cm_data.freeCellsp = cellp;
+}
+
 cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
 {
     cm_cell_t *cp, *cp2;
     long code;
-    char fullname[200]="";
+    char fullname[CELL_MAXNAMELEN]="";
     int  hasWriteLock = 0;
+    int  hasMutex = 0;
     afs_uint32 hash;
     cm_cell_rock_t rock;
 
@@ -150,7 +171,8 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
     lock_ObtainRead(&cm_cellLock);
     for (cp = cm_data.cellNameHashTablep[hash]; cp; cp=cp->nameNextp) {
         if (cm_stricmp_utf8(namep, cp->name) == 0) {
-            strcpy(fullname, cp->name);
+            strncpy(fullname, cp->name, CELL_MAXNAMELEN);
+            fullname[CELL_MAXNAMELEN-1] = '\0';
             break;
         }
     }
@@ -158,18 +180,18 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
     if (!cp) {
         for (cp = cm_data.allCellsp; cp; cp=cp->allNextp) {
             if (strnicmp(namep, cp->name, strlen(namep)) == 0) {
-                strcpy(fullname, cp->name);
+                strncpy(fullname, cp->name, CELL_MAXNAMELEN);
+                fullname[CELL_MAXNAMELEN-1] = '\0';
                 break;
             }
         }   
     }
 
-    lock_ReleaseRead(&cm_cellLock);
-
     if (cp) {
+        lock_ReleaseRead(&cm_cellLock);
         cm_UpdateCell(cp, flags);
     } else if (flags & CM_FLAG_CREATE) {
-        lock_ObtainWrite(&cm_cellLock);
+        lock_ConvertRToW(&cm_cellLock);
         hasWriteLock = 1;
 
         /* when we dropped the lock the cell could have been added
@@ -177,7 +199,8 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
          */
         for (cp = cm_data.cellNameHashTablep[hash]; cp; cp=cp->nameNextp) {
             if (cm_stricmp_utf8(namep, cp->name) == 0) {
-                strcpy(fullname, cp->name);
+                strncpy(fullname, cp->name, CELL_MAXNAMELEN);
+                fullname[CELL_MAXNAMELEN-1] = '\0';
                 break;
             }
         }   
@@ -187,24 +210,48 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
 
         for (cp = cm_data.allCellsp; cp; cp=cp->allNextp) {
             if (strnicmp(namep, cp->name, strlen(namep)) == 0) {
-                strcpy(fullname, cp->name);
+                strncpy(fullname, cp->name, CELL_MAXNAMELEN);
+                fullname[CELL_MAXNAMELEN-1] = '\0';
                 break;
             }
         }   
 
-        if (cp)
+        if (cp) {
+            lock_ObtainMutex(&cp->mx);
+            cm_AddCellToNameHashTable(cp);
+            cm_AddCellToIDHashTable(cp);           
+            lock_ReleaseMutex(&cp->mx);
             goto done;
+        }
 
-        if ( cm_data.currentCells >= cm_data.maxCells )
-            osi_panic("Exceeded Max Cells", __FILE__, __LINE__);
+        if ( cm_data.freeCellsp != NULL ) {
+            cp = cm_data.freeCellsp;
+            cm_data.freeCellsp = cp->freeNextp;
+
+            /* 
+             * The magic, cellID, and mx fields are already set.
+             */
+        } else {
+            if ( cm_data.currentCells >= cm_data.maxCells )
+                osi_panic("Exceeded Max Cells", __FILE__, __LINE__);
+
+            /* don't increment currentCells until we know that we 
+             * are going to keep this entry 
+             */
+            cp = &cm_data.cellBaseAddress[cm_data.currentCells];
+            memset(cp, 0, sizeof(cm_cell_t));
+            cp->magic = CM_CELL_MAGIC;
+
+            /* the cellID cannot be 0 */
+            cp->cellID = ++cm_data.currentCells;
+
+            /* otherwise we found the cell, and so we're nearly done */
+            lock_InitializeMutex(&cp->mx, "cm_cell_t mutex", LOCK_HIERARCHY_CELL);
+        }
+
+        lock_ReleaseWrite(&cm_cellLock);
+        hasWriteLock = 0;
 
-        /* don't increment currentCells until we know that we 
-         * are going to keep this entry 
-         */
-        cp = &cm_data.cellBaseAddress[cm_data.currentCells];
-        memset(cp, 0, sizeof(cm_cell_t));
-        cp->magic = CM_CELL_MAGIC;
-        
         rock.cellp = cp;
         rock.flags = flags;
         code = cm_SearchCellFile(namep, fullname, cm_AddCellProc, &rock);
@@ -220,21 +267,27 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
                 if ( code ) {
                     osi_Log3(afsd_logp,"in cm_GetCell_gen cm_SearchCellByDNS(%s) returns code= %d fullname= %s", 
                              osi_LogSaveString(afsd_logp,namep), code, osi_LogSaveString(afsd_logp,fullname));
+                    cm_FreeCell(cp);
                     cp = NULL;
                     goto done;
                 } else {   /* got cell from DNS */
+                    lock_ObtainMutex(&cp->mx);
+                    hasMutex = 1;
                     cp->flags |= CM_CELLFLAG_DNS;
                     cp->flags &= ~CM_CELLFLAG_VLSERVER_INVALID;
                     cp->timeout = time(0) + ttl;
                 }
-            } else {
+            } 
+            else 
 #endif
+            {
+                cm_FreeCell(cp);
                 cp = NULL;
                 goto done;
-#ifdef AFS_AFSDB_ENV
 	    }
-#endif
         } else {
+            lock_ObtainMutex(&cp->mx);
+            hasMutex = 1;
 	    cp->timeout = time(0) + 7200;	/* two hour timeout */
 	}
 
@@ -251,26 +304,30 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
         }   
 
         if (cp2) {
-            cm_FreeServerList(&cp->vlServersp, CM_FREESERVERLIST_DELETE);
+            if (hasMutex) {
+                lock_ReleaseMutex(&cp->mx);
+                hasMutex = 0;
+            }
+            cm_FreeCell(cp);
             cp = cp2;
             goto done;
         }
 
-
         /* randomise among those vlservers having the same rank*/ 
         cm_RandomizeServer(&cp->vlServersp);
 
-        /* otherwise we found the cell, and so we're nearly done */
-        lock_InitializeMutex(&cp->mx, "cm_cell_t mutex");
-
+        if (!hasMutex)
+            lock_ObtainMutex(&cp->mx);
         /* copy in name */
         strncpy(cp->name, fullname, CELL_MAXNAMELEN);
         cp->name[CELL_MAXNAMELEN-1] = '\0';
 
-        /* the cellID cannot be 0 */
-        cp->cellID = ++cm_data.currentCells;
+        cm_AddCellToNameHashTable(cp);
+        cm_AddCellToIDHashTable(cp);           
+        lock_ReleaseMutex(&cp->mx);
+        hasMutex = 0;
 
-		/* append cell to global list */
+        /* append cell to global list */
         if (cm_data.allCellsp == NULL) {
             cm_data.allCellsp = cp;
         } else {
@@ -280,18 +337,24 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags)
         }
         cp->allNextp = NULL;
 
-        cm_AddCellToNameHashTable(cp);
-        cm_AddCellToIDHashTable(cp);           
+    } else {
+        lock_ReleaseRead(&cm_cellLock);
     }
-
   done:
+    if (hasMutex && cp)
+        lock_ReleaseMutex(&cp->mx);
     if (hasWriteLock)
         lock_ReleaseWrite(&cm_cellLock);
     
     /* fullname is not valid if cp == NULL */
-    if (cp && newnamep)
-        strcpy(newnamep, fullname);
-    
+    if (newnamep) {
+        if (cp) {
+            strncpy(newnamep, fullname, CELL_MAXNAMELEN);
+            newnamep[CELL_MAXNAMELEN-1]='\0';
+        } else {
+            newnamep[0] = '\0';
+        }
+    }
     return cp;
 }
 
@@ -336,10 +399,19 @@ cm_ValidateCell(void)
         }
     }
 
+    for (cellp = cm_data.freeCellsp; cellp; cellp=cellp->freeNextp, count++) {
+        if ( count != 0 && cellp == cm_data.freeCellsp ||
+             count > cm_data.maxCells ) {
+            afsi_log("cm_ValidateCell failure: cm_data.freeCellsp infinite loop");
+            fprintf(stderr, "cm_ValidateCell failure: cm_data.freeCellsp infinite loop\n");
+            return -3;
+        }
+    }
+
     if ( count != cm_data.currentCells ) {
         afsi_log("cm_ValidateCell failure: count != cm_data.currentCells");
         fprintf(stderr, "cm_ValidateCell failure: count != cm_data.currentCells\n");
-        return -3;
+        return -4;
     }
     
     return 0;
@@ -365,7 +437,7 @@ void cm_InitCell(int newFile, long maxCells)
     if (osi_Once(&once)) {
         cm_cell_t * cellp;
 
-        lock_InitializeRWLock(&cm_cellLock, "cell global lock");
+        lock_InitializeRWLock(&cm_cellLock, "cell global lock", LOCK_HIERARCHY_CELL_GLOBAL);
 
         if ( newFile ) {
             cm_data.allCellsp = NULL;
@@ -383,10 +455,11 @@ void cm_InitCell(int newFile, long maxCells)
             memset(cellp, 0, sizeof(cm_cell_t));
             cellp->magic = CM_CELL_MAGIC;
 
-            lock_InitializeMutex(&cellp->mx, "cm_cell_t mutex");
+            lock_InitializeMutex(&cellp->mx, "cm_cell_t mutex", LOCK_HIERARCHY_CELL);
 
             /* copy in name */
             strncpy(cellp->name, "Freelance.Local.Cell", CELL_MAXNAMELEN); /*safe*/
+            cellp->name[CELL_MAXNAMELEN-1] = '\0';
 
             /* thread on global list */
             cellp->allNextp = cm_data.allCellsp;
@@ -396,12 +469,14 @@ void cm_InitCell(int newFile, long maxCells)
             cellp->vlServersp = NULL;
             cellp->flags = CM_CELLFLAG_FREELANCE;
 
-	    cm_AddCellToNameHashTable(cellp);
-	    cm_AddCellToIDHashTable(cellp);           
+            lock_ObtainMutex(&cellp->mx);
+            cm_AddCellToNameHashTable(cellp);
+            cm_AddCellToIDHashTable(cellp);           
+            lock_ReleaseMutex(&cellp->mx);
 #endif  
         } else {
             for (cellp = cm_data.allCellsp; cellp; cellp=cellp->allNextp) {
-                lock_InitializeMutex(&cellp->mx, "cm_cell_t mutex");
+                lock_InitializeMutex(&cellp->mx, "cm_cell_t mutex", LOCK_HIERARCHY_CELL);
                 cellp->vlServersp = NULL;
                 cellp->flags |= CM_CELLFLAG_VLSERVER_INVALID;
             }
@@ -442,8 +517,8 @@ int cm_DumpCells(FILE *outputFile, char *cookie, int lock)
     WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
 
     for (cellp = cm_data.allCellsp; cellp; cellp=cellp->allNextp) {
-        sprintf(output, "%s cellp=0x%p,name=%s ID=%d flags=0x%x\r\n", 
-                cookie, cellp, cellp->name, cellp->cellID, cellp->flags);
+        sprintf(output, "%s cellp=0x%p,name=%s ID=%d flags=0x%x timeout=%I64u\r\n", 
+                cookie, cellp, cellp->name, cellp->cellID, cellp->flags, cellp->timeout);
         WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
     }
 
