@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_misc.c,v 1.44.4.3 2006/07/31 21:27:39 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_misc.c,v 1.44.4.6 2008/07/01 03:35:23 shadow Exp $");
 
 #include <linux/module.h> /* early to avoid printf->printk mapping */
 #include "afs/sysincludes.h"
@@ -65,8 +65,6 @@ afs_osi_SetTime(osi_timeval_t * tvp)
 #endif
 }
 
-struct task_struct *rxk_ListenerTask;
-
 void
 osi_linux_mask(void)
 {
@@ -76,13 +74,6 @@ osi_linux_mask(void)
     SIG_UNLOCK(current);
 }
 
-void
-osi_linux_rxkreg(void)
-{
-    rxk_ListenerTask = current;
-}
-
-
 #if defined(AFS_LINUX24_ENV)
 /* LOOKUP_POSITIVE is becoming the default */
 #ifndef LOOKUP_POSITIVE
@@ -108,10 +99,17 @@ osi_lookupname_internal(char *aname, int followlink, struct vfsmount **mnt,
 #endif
 
     if (!code) {
+#if defined(STRUCT_NAMEIDATA_HAS_PATH)
+	*dpp = dget(nd.path.dentry);
+        if (mnt)
+	    *mnt = mntget(nd.path.mnt);
+	path_put(&nd.path);
+#else
 	*dpp = dget(nd.dentry);
         if (mnt)
            *mnt = mntget(nd.mnt);
 	path_release(&nd);
+#endif
     }
     return code;
 }
@@ -170,7 +168,7 @@ int osi_abspath(char *aname, char *buf, int buflen,
 		int followlink, char **pathp)
 {
     struct dentry *dp = NULL;
-    struct vfsmnt *mnt = NULL;
+    struct vfsmount *mnt = NULL;
     char *tname, *path;
     int code;
 
@@ -180,7 +178,12 @@ int osi_abspath(char *aname, char *buf, int buflen,
 	return -PTR_ERR(tname);
     code = osi_lookupname_internal(tname, followlink, &mnt, &dp);   
     if (!code) {
+#if defined(D_PATH_TAKES_STRUCT_PATH)
+	struct path p = { mnt, dp };
+	path = d_path(&p, buf, buflen);
+#else
 	path = d_path(dp, mnt, buf, buflen);
+#endif
 
 	if (IS_ERR(path)) {
 	    code = -PTR_ERR(path);
