@@ -14,7 +14,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_analyze.c,v 1.22.14.6.2.1 2008/05/23 14:25:34 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_analyze.c,v 1.22.14.9 2008/08/22 18:52:57 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -247,20 +247,21 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
     afs_int32 i;
     afs_int32 serversleft = 0;
 
-    if (afid)
+    if (afid) {
 	tvp = afs_FindVolume(afid, READ_LOCK);
-    if (tvp) {
-	for (i = 0; i < MAXHOSTS; i++) {
-	    if (tvp->serverHost[i] == tsp) {
-		areq->skipserver[i] = 1;
-	    }
-	    if (tvp->serverHost[i] &&
-		!(tvp->serverHost[i]->addr->sa_flags & 
-		  SRVR_ISDOWN)) {
-		areq->skipserver[i] = 1;
+	if (tvp) {
+	    for (i = 0; i < MAXHOSTS; i++) {
+		if (tvp->serverHost[i] == tsp) {
+		    areq->skipserver[i] = 1;
+		}
+		if (tvp->serverHost[i] &&
+		    !(tvp->serverHost[i]->addr->sa_flags & 
+		      SRVR_ISDOWN)) {
+		    areq->skipserver[i] = 1;
+		}
 	    }
+	    afs_PutVolume(tvp, READ_LOCK);
 	}
-	afs_PutVolume(tvp, READ_LOCK);
     }
     for (i = 0; i < MAXHOSTS; i++) {
 	if (areq->skipserver[i] == 0) {
@@ -381,7 +382,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	return shouldRetry;	/* should retry */
     }
 
-    if (!aconn) {
+    if (!aconn || !aconn->srvr) {
 	if (!areq->volumeError) {
 	    if (aerrP)
 		(aerrP->err_Network)++;
