@@ -24,20 +24,17 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
+#ifdef AFS_NT40_ENV
+#include <windows.h>
+#include <rpc.h>
+#endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/xstat/xstat_fs_callback.c,v 1.13 2003/07/15 23:17:52 shadow Exp $");
+    ("$Header: /cvs/openafs/src/xstat/xstat_fs_callback.c,v 1.15.2.1 2007/10/30 15:16:59 shadow Exp $");
 
 #include <errno.h>
 #include <stdio.h>		/*Standard I/O stuff */
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/afscbint.h>	/*Callback interface defs */
 
@@ -54,7 +51,11 @@ init_afs_cb()
 {
     int count;
 
+#ifdef AFS_NT40_ENV
+    UuidCreate((UUID *)&afs_cb_interface.uuid);
+#else
     afs_uuid_create(&afs_cb_interface.uuid);
+#endif
     count = rx_getAllAddr(&afs_cb_interface.addr_in, AFS_MAX_INTERFACE_ADDR);
     if (count <= 0)
 	afs_cb_interface.numberOfInterfaces = 0;
@@ -781,3 +782,15 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call * rxcall,
      */
     return (0);
 }
+
+int SRXAFSCB_GetDE(a_call, a_index, addr, inode, flags, time, fileName)
+     struct rx_call *a_call;
+     afs_int32 a_index;
+     afs_int32 addr;
+     afs_int32 inode;
+     afs_int32 flags;
+     afs_int32 time;
+     char ** fileName;
+{
+    return RXGEN_OPCODE;
+}
